#include "Automat.h"
#include "dcMotor.h"
#include "PWM.h"
#include <stdlib.h>

#define OPEN_LID(T) changePWM(LID_M,80);_delay_ms(T)
#define CLOSE_LID(T) changePWM(LID_M,180);_delay_ms(T)
#define SET_LID(VAL,T) changePWM(LID_M,(VAL));_delay_ms(T)

#define PRESS_BTN(T) changePWM(ARM_M,196);_delay_ms(T)
#define BRING_ARM_INSIDE(T) changePWM(ARM_M,100);_delay_ms(T)
#define SET_ARM(VAL,T) changePWM(ARM_M,(VAL));_delay_ms(T)

#define SLOW_ITERATIONS 24
#define SLOW_SPEED 4


enum STRATEGY {
	SGY_INIT,
	SGY_SLOW_RETREAT,
	SGY_SLOW_RETREAT_REENGAGE,
	SGY_STANDBY,
	SGY_RUN_AWAY,
	SGY_FAST_ACT,
	SGY_FEIGN,
	SGY_DEFAULT = -1
};

#define AWAIT_EVENT(_EVENT, _TIME) \
for(unsigned int k = 0, _time = (_TIME) * 4; !(_EVENT); ++k) { \
	_delay_us(250); \
	if (k >= _time)	{ \
		state = SGY_STANDBY; \
		CLOSE_LID(1); \
		return; \
	} \
}

extern volatile short switchValue;

//Behavioural Automate
void Act() {
	static enum STRATEGY state = SGY_INIT;
	short i = 0;
	short p;
	short j = 0;
	switch(state) {
		case SGY_INIT:
			while (!switchValue);
			srand(TCNT1L);
			OPEN_LID(500);
			PRESS_BTN(500);
			BRING_ARM_INSIDE(500);
			CLOSE_LID(100);
			state = SGY_SLOW_RETREAT;
			return;

		case SGY_SLOW_RETREAT:
			AWAIT_EVENT(switchValue, 5000);
			OPEN_LID(750);
			PRESS_BTN(250);
			for (i = 1; i < SLOW_ITERATIONS; ++i)
			{
				if (switchValue) {
					state = SGY_SLOW_RETREAT_REENGAGE;
					return;
				}
				SET_ARM(196 - SLOW_SPEED * i, 50);
			}
			BRING_ARM_INSIDE(50);
			CLOSE_LID(100);
			p = rand() % 100;
			if (p >= 60) {
				state = SGY_DEFAULT;
			} else if (p >= 30) {
				state = SGY_FEIGN;
			}
			return;

		case SGY_SLOW_RETREAT_REENGAGE:
			while (i < SLOW_ITERATIONS) {
				PRESS_BTN(250);
				for (i = 1; i < SLOW_ITERATIONS; ++i)
				{
					if (switchValue) {
						break;
					}
					SET_ARM(196 - SLOW_SPEED * i, 50);
				}
				if (++j >= 5) {
					state = SGY_FAST_ACT;
					return;
				}
			}
			BRING_ARM_INSIDE(50);
			CLOSE_LID(100);

			p = rand() % 100;
			if (p > 30) {
				state = SGY_STANDBY;
			} else {
				state = SGY_RUN_AWAY;
			}
			return;

		case SGY_STANDBY:
			BRING_ARM_INSIDE(0);
			CLOSE_LID(0);
			while (!switchValue && !PIR_VAL);
			if (switchValue) {
				state = SGY_DEFAULT;
				return;
			}
			SET_LID(140, 200);
			for (i = 0; i < 50; ++i)
			{
				if (switchValue) {
					state = SGY_DEFAULT;
					return;
				}
				_delay_ms(75);
			}
			CLOSE_LID(100);
			state = SGY_DEFAULT;
			return;

		case SGY_FAST_ACT:
			AWAIT_EVENT(switchValue, 2000);
			PRESS_BTN(150);
			SET_ARM(170, 120);
			return;

		case SGY_FEIGN:
			SET_LID(170, 0);
			SET_ARM(180, 750);
			BRING_ARM_INSIDE(100);
			CLOSE_LID(100);

			p = rand() % 100;
			if (p >= 50) {
				state = SGY_DEFAULT;
			} else {
				state = SGY_SLOW_RETREAT;
			}
			return;

		case SGY_RUN_AWAY:
			while (!PIR_VAL);
			setDcMovement(1 << (rand() % 2));
			_delay_ms(2000);
			setDcMovement(0);
			state = SGY_DEFAULT;
			return;

		default:
			AWAIT_EVENT(switchValue, 5000);
			OPEN_LID(500);
			PRESS_BTN(500);
			BRING_ARM_INSIDE(100);
			CLOSE_LID(200);
			state = SGY_SLOW_RETREAT;
			return;
	}
}
