#include "PWM.h"
#include "dcMotor.h"

volatile short motor[MAX_PWM];
volatile int delay[MAX_PWM];
volatile int active[MAX_PWM];

// wait ~= time*10 microseconds
static void _dynamic_wait(int time) {
	while (time--) {
		_delay_us(10);
	}
}

// set the current value of a PWM
static void tickPWM(short m, short on) {
	if (m >= 0) m = motor[m];
	switch(m) {
case LID_M: // lid
	if (on) {
		PORTD |= (1 << PD4);
	} else {
		PORTD &= ~(1 << PD4);
	}
	break;
case ARM_M: // arm
	if (on) {
		PORTD |= (1 << PD6);
	} else {
		PORTD &= ~(1 << PD6);
	}
	break;
case ELE_MOV_M: // plaform move
	if (on) {
		PORTD |= (1 << PD5);
	} else {
		PORTD &= ~(1 << PD5);
	}
	break;
case ELE_ROT_M: // has Led, platform rotate
	if (on) {
		PORTD |= (1 << PD7);
	} else {
		PORTD &= ~(1 << PD7);
	}
	break;
case 4:
	if (on) {
		PORTB |= (1 << PB3);
	} else {
		PORTB &= ~(1 << PB3);
	}
	break;
case 5:
	if (on) {
		PORTB |= (1 << PB4);
	} else {
		PORTB &= ~(1 << PB4);
	}
	break;
default:
	if (on) {
		PORTD |= (0xf << PD4);
		//PORTB |= (0x3 << PB3);
	} else {
		PORTD &= ~(0xf << PD4);
		//PORTB &= ~(0x3 << PB3);
	}
	break;
	}
}

void initPWM() {
	short i;
	for (i = 0; i < MAX_PWM; ++i) {
		delay[i] = 1;
		motor[i] = i;
	}

	// activate only 4 pins for output
    DDRD |= (0xf << PD4);
	//DDRB |= (0x3 << PB3);

	// set frequency to 50Hz (20ms)
    ICR1 = 19999;
    TCCR1A = 0;
    TCCR1B = (0x2 << CS10) | (0x2 << WGM12);

	// enable interrupt for software PWM
	TIMSK1 = (1 << TOIE1);
}

void doPWM() {
	int i;

	// set all PWMs output to 1
	tickPWM(-1, 1);

	// one by one stop the PWMs using the queue
	for (i = 0; i < MAX_PWM; ++i) {
		_dynamic_wait(delay[i]);
		tickPWM(i, 0);
	}
}

void changePWM(short m, int d) {
	short _motor[MAX_PWM];
	int _delay[MAX_PWM];
	short i;

	// parse the PWM queue
	_delay[0] = delay[0];
	_motor[0] = motor[0];
	for (i = 1; i < MAX_PWM; ++i) {
		_delay[i] = delay[i] + _delay[i-1];
		_motor[i] = motor[i];
	}

	// identify what motor has it's dutycycle updated
	for (i = 0; i < MAX_PWM; ++i) {
		if (_motor[i] == m) {
			break;
		}
	}

	if (_delay[i] > d) {
	// if Duty has shrinked
	// try to bring the PWM earlier in queue
		for (; i > 0; --i) {
			if (_delay[i-1] > d) {
				_delay[i] = _delay[i-1];
				_motor[i] = _motor[i-1];
			} else {
				break;
			}
		}
		_delay[i] = d;
		_motor[i] = m;

	} else {
	// if Duty has grown
	// try to bring the PWM later in queue
		for (; i < MAX_PWM-1; ++i) {
			if (_delay[i+1] < d) {
				_delay[i] = _delay[i+1];
				_motor[i] = _motor[i+1];
			} else {
				break;
			}
		}
		_delay[i] = d;
		_motor[i] = m;
	}

	// disable interrupts
	cli();
	// update the PWM queue
	motor[0] = _motor[0];
	delay[0] = _delay[0];
	for (i = 1; i < MAX_PWM; ++i) {
		motor[i] = _motor[i];
		delay[i] = _delay[i] - _delay[i-1];
	}
	sei();
	// enable interrupts
}

